local Vector3 = require "types.vector3"

---@class AABB
local AABB = {
  min = Vector3(),
  max = Vector3(),
}

local RED = {
  r = 255,
  g = 0,
  b = 0,
  a = 255,
}

setmetatable(AABB, AABB)

AABB.__index = AABB

---@param min vectorlike
---@param max vectorlike
---@return AABB
function AABB.new(min, max)
  min = min or Vector3()
  max = max or Vector3()
  local aabb = {
    min = min,
    max = max,
  }

  return setmetatable(aabb, AABB)
end

function AABB:draw()
  -- bottom rectangle
  draw_line_3d{start = self.min, finish = Vector3(self.max.x, self.min.y, self.min.z)}
  draw_line_3d{start = self.min, finish = Vector3(self.min.x, self.min.y, self.max.z)}
  draw_line_3d{start = Vector3(self.max.x, self.min.y, self.max.z), finish = Vector3(self.min.x, self.min.y, self.max.z)}
  draw_line_3d{start = Vector3(self.max.x, self.min.y, self.max.z), finish = Vector3(self.max.x, self.min.y, self.min.z)}
  -- bottom rectangle diagonal
  draw_line_3d{start = self.min, finish = Vector3(self.max.x, self.min.y, self.max.z), color = RED}
  -- top rectangle
  draw_line_3d{start = Vector3(self.min.x, self.max.y, self.min.z), finish = Vector3(self.max.x, self.max.y, self.min.z)}
  draw_line_3d{start = Vector3(self.min.x, self.max.y, self.min.z), finish = Vector3(self.min.x, self.max.y, self.max.z)}
  draw_line_3d{start = Vector3(self.max.x, self.max.y, self.max.z), finish = Vector3(self.min.x, self.max.y, self.max.z)}
  draw_line_3d{start = Vector3(self.max.x, self.max.y, self.max.z), finish = Vector3(self.max.x, self.max.y, self.min.z)}
  -- top rectangle diagonal
  draw_line_3d{start = Vector3(self.max.x, self.max.y, self.max.z), finish = Vector3(self.min.x, self.max.y, self.min.z), color = RED}
  -- hull
  draw_line_3d{start = self.min, finish = Vector3(self.min.x, self.max.y, self.min.z)}
  draw_line_3d{start = self.min, finish = Vector3(self.max.x, self.max.y, self.min.z), color = RED}
  
  draw_line_3d{start = Vector3(self.max.x, self.min.y, self.min.z), finish = Vector3(self.max.x, self.max.y, self.min.z)}
  draw_line_3d{start = Vector3(self.max.x, self.min.y, self.min.z), finish = Vector3(self.max.x, self.max.y, self.max.z), color = RED}
  
  draw_line_3d{start = Vector3(self.max.x, self.min.y, self.max.z), finish = Vector3(self.max.x, self.max.y, self.max.z)}
  draw_line_3d{start = Vector3(self.max.x, self.min.y, self.max.z), finish = Vector3(self.min.x, self.max.y, self.max.z), color = RED}

  draw_line_3d{start = Vector3(self.min.x, self.min.y, self.max.z), finish = Vector3(self.min.x, self.max.y, self.max.z)}
  draw_line_3d{start = Vector3(self.min.x, self.min.y, self.max.z), finish = Vector3(self.min.x, self.max.y, self.min.z), color = RED}

end

return AABB
